#!/bin/sh
NFS_EXPORTS=/mnt/base/etc/exports
EXPORTFS=/usr/app/nfs/sbin/exportfs
HOSTS_ALLOW=/etc/hosts.allow
HOSTS_DENY=/etc/hosts.deny
SMB_CONFIG=/opt/etc/samba/smb.conf
SQLITE3=/opt/bin/sqlite3
NAS_DB=/mnt/base/nasdb
SERVICE=
DIR=
ACCESS="rw"
HOSTS=

if [ -z $1 ];then
	echo "Usage:$0 service=[nfs|samba] dir=[dirctory] hosts=\"IP|Domainname\""
	exit 1
fi


for VAL in $@
do
	SERVICE_PREFIX=${VAL:0:7}
	DIR_PREFIX=${VAL:0:3}
	HOST_PREFIX=${VAL:0:5}
	ACCESS_PREFIX=${VAL:0:6}	
	
	if [ XX$SERVICE_PREFIX == XXservice ];then
		SERVICE=`echo $VAL | awk -F'service=' '{print $2}'`
		continue			
	fi	
	if [ XX$DIR_PREFIX == XXdir ];then
		DIR=`echo $VAL | awk -F'dir=' '{print $2}'`
		continue
	fi
	if [ XX$HOST_PREFIX == XXhosts ];then
		HOSTS=`echo $VAL | awk -F'hosts=' '{print $2}'`
		continue
	fi
	if [ XX$ACCESS_PREFIX == XXaccess ];then
		ACCESS=`echo $VAL | awk -F'access=' '{print $2}'`
		continue
	fi
done
echo service=$SERVICE dir=$DIR hosts=$HOSTS access=$ACCESS

if [ "$SERVICE" == "nfs" ]; then
	if [ ! -f ${NFS_EXPORTS} ];then
		rm -rf ${NFS_EXPORTS}
	fi

	MNT_PATH=`${SQLITE3} ${NAS_DB} "select mntpath from share where foldername = '${DIR}'"`
	#echo MNT_PATH:$MNT_PATH
	MNT_PATH="\"$MNT_PATH\""
	
	#Get the match line number,then delete the line.
	sed -n "/^[$MNT_PATH]/=" $NFS_EXPORTS | while read LINE
	do
		#echo $LINE #Mactch line
		dir=`awk "NR==$LINE" $NFS_EXPORTS | awk '{print $1}'`
		if [ "$dir" == "$MNT_PATH" ];then
			echo "Update line $LINE..."   #Delete line
			sed -i "${LINE}d" $NFS_EXPORTS
		fi
	done
	
	if [ "XX$ACCESS" == "XXno"  ];then
		exit 1
	fi

	#Insert a new line at end of file.
	HOST_COUNT=`echo "$HOSTS" | awk -F',' '{print NF}'`
	info=0
	echo "$HOSTS" | awk -F',' 'BEGIN{i=1}{while(1){if($i=="")break;print $i;i++}}' | while read HOST
	do	
		info=`expr $info + 1`
		STRING="$HOST(${ACCESS},sync,no_root_squash)"
		#echo STRING:$STRING
		STRING_LONG="$STRING $STRING_LONG"
		
		if [ "$info" -eq "$HOST_COUNT" ];then
			#echo LONG:$STRING_LONG
			echo -e "$MNT_PATH\t$STRING_LONG" >> $NFS_EXPORTS
		fi
	done

	#At last,flush export directorys
	$EXPORTFS -ar > /dev/null 2>&1
elif [ "$SERVICE" == "samba" ]; then
	if [ "$ACCESS" = "rw" ]; then
		ACCESS=2
	elif [ "$ACCESS" == "ro" ]; then
		ACCESS=1
	elif [ "$ACCESS" == "no" ];then
		ACCESS=0
	fi
	
	#Generate random string as hash value
	HASH=`dd if=/dev/urandom bs=1 count=200 2>/dev/null | awk 'BEGIN{FS="\n";OFS="";ORS=""} {print $0}' | sed 's/[^a-zA-Z0-9]//g'`
	FOLDER_ID=`${SQLITE3} $NAS_DB "select folder_id from share where foldername = '$DIR'"`
	echo HASH:$HASH
	#echo FOLDER_ID:$FOLDER_ID

	#Insert into database
	${SQLITE3} ${NAS_DB} "delete from acl_host where folder_id = $FOLDER_ID"
	${SQLITE3} ${NAS_DB} "insert into acl_host values ('$HOSTS',$FOLDER_ID,'$SERVICE',$ACCESS,'$HASH','')"
	
	if [ ! -f ${SMB_CONFIG} ]; then
		rm -rf ${SMB_CONFIG}
	fi
	
	if [ "XX$HOSTS" != "XX*" ];then
		#Get samba 'share dir' line number
		#Note:$N number must range from between two lines which including "inherit permission" 
		N=20
		LINE=`grep -n -A $N "\[${DIR}\]"  $SMB_CONFIG | grep "inherit permissions" | awk -F'-' '{print $1}'`
		#echo $LINE

		#Delete hosts limit in $DIR sectioni(between '[]' and '[]').
		sed -i "/\[$DIR\]/,/\[/{/hosts allow/,/hosts deny/d}" $SMB_CONFIG
	
		#Add hosts limit in $DIR section.
		[ ! -z "$LINE" ] && sed -i "$LINE a\hosts allow = $HOSTS\nhosts deny = 0.0.0.0/0" $SMB_CONFIG
	else
		sed -i "/\[$DIR\]/,/\[/{/hosts allow/,/hosts deny/d}" $SMB_CONFIG
	fi
	
	service samba restart
fi

